@@ -1,5 +1,5 @@
 /* BFD support for the M32R processor.
-   Copyright (C) 1996-2021 Free Software Foundation, Inc.
+   Copyright (C) 1996-2024 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -31,9 +31,9 @@
 
 static const bfd_arch_info_type arch_info_struct[] =
 {
-  N (bfd_mach_m32rx, "m32rx", FALSE, M32R2_NEXT) ,
-  N (bfd_mach_m32r2, "m32r2", FALSE, NULL)
+  N (bfd_mach_m32rx, "m32rx", false, M32R2_NEXT) ,
+  N (bfd_mach_m32r2, "m32r2", false, NULL)
 };
 
 const bfd_arch_info_type bfd_m32r_arch =
-  N (bfd_mach_m32r, "m32r", TRUE, NEXT);
+  N (bfd_mach_m32r, "m32r", true, NEXT);
