@@ -1,5 +1,5 @@
 /* PPC64 ELF support for BFD.
-   Copyright (C) 2003-2019 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -158,10 +158,11 @@ START_RELOC_NUMBERS (elf_ppc64_reloc_type)
   RELOC_NUMBER (R_PPC64_PLTSEQ,		   119)
   RELOC_NUMBER (R_PPC64_PLTCALL,	   120)
 
-/* Powerxx support.  */
+/* Power10 support.  */
   RELOC_NUMBER (R_PPC64_PLTSEQ_NOTOC,	   121)
   RELOC_NUMBER (R_PPC64_PLTCALL_NOTOC,	   122)
   RELOC_NUMBER (R_PPC64_PCREL_OPT,	   123)
+  RELOC_NUMBER (R_PPC64_REL24_P9NOTOC,	   124)
 
   RELOC_NUMBER (R_PPC64_D34,		   128)
   RELOC_NUMBER (R_PPC64_D34_LO,		   129)
@@ -181,6 +182,12 @@ START_RELOC_NUMBERS (elf_ppc64_reloc_type)
   RELOC_NUMBER (R_PPC64_REL16_HIGHESTA34,  143)
   RELOC_NUMBER (R_PPC64_D28,		   144)
   RELOC_NUMBER (R_PPC64_PCREL28,	   145)
+  RELOC_NUMBER (R_PPC64_TPREL34,	   146)
+  RELOC_NUMBER (R_PPC64_DTPREL34,	   147)
+  RELOC_NUMBER (R_PPC64_GOT_TLSGD_PCREL34, 148)
+  RELOC_NUMBER (R_PPC64_GOT_TLSLD_PCREL34, 149)
+  RELOC_NUMBER (R_PPC64_GOT_TPREL_PCREL34, 150)
+  RELOC_NUMBER (R_PPC64_GOT_DTPREL_PCREL34, 151)
 
 #ifndef RELOC_MACROS_GEN_FUNC
 /* Relocation only used internally by gas or ld.  If you need to use
@@ -218,8 +225,8 @@ END_RELOC_NUMBERS (R_PPC64_max)
 
 #define IS_PPC64_TLS_RELOC(R)						\
   (((R) >= R_PPC64_TLS && (R) <= R_PPC64_DTPREL16_HIGHESTA)		\
-   || ((R) >= R_PPC64_TPREL16_HIGH && (R) <= R_PPC64_DTPREL16_HIGHA))
-
+   || ((R) >= R_PPC64_TPREL16_HIGH && (R) <= R_PPC64_DTPREL16_HIGHA)	\
+   || ((R) >= R_PPC64_TPREL34 && (R) <= R_PPC64_GOT_DTPREL_PCREL34))
 
 /* e_flags bits specifying ABI.
    1 for original function descriptor using ABI,
@@ -250,14 +257,14 @@ END_RELOC_NUMBERS (R_PPC64_max)
 #define STO_PPC64_LOCAL_BIT		5
 #define STO_PPC64_LOCAL_MASK		(7 << STO_PPC64_LOCAL_BIT)
 
-// 3 bit other field to bytes.
+/* 3 bit other field to bytes.  */
 static inline unsigned int
 ppc64_decode_local_entry(unsigned int other)
 {
   return ((1 << other) >> 2) << 2;
 }
 
-// bytes to field value.
+/* bytes to field value.  */
 static inline unsigned int
 ppc64_encode_local_entry(unsigned int val)
 {
